#!/usr/bin/env python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# +------------------------------------------------------------------+
# |             ____ _               _        __  __ _  __           |
# |            / ___| |__   ___  ___| | __   |  \/  | |/ /           |
# |           | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /            |
# |           | |___| | | |  __/ (__|   <    | |  | | . \            |
# |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
# |                                                                  |
# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
# +------------------------------------------------------------------+
#
# This file is part of Check_MK.
# The official homepage is at http://mathias-kettner.de/check_mk.
#
# check_mk is free software;  you can redistribute it and/or modify it
# under the  terms of the  GNU General Public License  as published by
# the Free Software Foundation in version 2.  check_mk is  distributed
# in the hope that it will be useful, but WITHOUT ANY WARRANTY;  with-
# out even the implied warranty of  MERCHANTABILITY  or  FITNESS FOR A
# PARTICULAR PURPOSE. See the  GNU General Public License for more de-
# tails. You should have  received  a copy of the  GNU  General Public
# License along with GNU Make; see the file  COPYING.  If  not,  write
# to the Free Software Foundation, Inc., 51 Franklin St,  Fifth Floor,
# Boston, MA 02110-1301 USA.

import abc

from livestatus import MKLivestatusNotFoundError
import cmk.gui.sites as sites
import cmk.gui.visuals as visuals
from cmk.gui.i18n import _
from cmk.gui.globals import html
from cmk.gui.htmllib import HTML

# from cmk.gui.plugins.visuals.utils import FilterCRESite
from cmk.gui.plugins.dashboard import (
    Dashlet,
    dashlet_registry,
)

from cmk.gui.valuespec import (
    Dictionary,
    Tuple,
    DropdownChoice,
    Integer,
)
from cmk.gui.plugins.wato.utils import (
    ContactGroupSelection,
    HostGroupSelection,
    ServiceGroupSelection,
    )
from cmk.gui.plugins.views import (
    RowTableLivestatus,
)

class DashStatelets(Dashlet):
    __metaclass__ = abc.ABCMeta

    @classmethod
    def is_resizable(cls):
        return True

    @classmethod
    def initial_size(cls):
        return (30, 18)

    @classmethod
    def initial_refresh_interval(cls):
        return 60

    @classmethod
    def _get_grp_state_ec(self, grp):
        cg, table, ack = grp
        statarr = []
        if ack:
            response = sites.live().query("GET eventconsoleevents\nColumns: event_id event_phase event_state event_contact_groups\nFilter: event_contact_groups_precedence = rule\nFilter: event_contact_groups >= %s\nAnd: 2\n" % cg)
        else:
            response = sites.live().query("GET eventconsoleevents\nColumns: event_id event_phase event_state event_contact_groups\nFilter: event_contact_groups_precedence = rule\nFilter: event_contact_groups >= %s\nFilter: event_phase != ack\nAnd: 3\n" % cg)
        for r in response:
            statarr.append(r[2])
        if len(statarr) > 0:
            return max(statarr)
        else:
            return 0

    @classmethod
    def _get_grp_state_hg(self, grp):
        statarr = []
        query = "GET %s\n%s\n%s\n" % ('hostgroups', "Columns: name", "")
        response = sites.live().query("GET hostgroups\nColumns: name worst_host_state worst_service_state\nFilter: name >= %s\n" % grp)
        if len(response) > 0:
            n, h, s = response[0]
            if  h > s:
                return h
            else:
                return s

    @classmethod
    def _get_grp_state_sg(self, grp):
        statarr = []
        query = "GET %s\n%s\n%s\n" % ('servicegroups', "Columns: name", "")
        response = sites.live().query("GET servicegroups\nColumns: name worst_service_state\nFilter: name >= %s\n" % grp)
        if len(response) > 0:
            n, s = response[0]
            return s

    def _colorpicker(self, state):
        sdict = {
            0: "Ok",
            1: "Warn",
            2: "Crit",
            3: "Unkn",
            99: "N/A",
            None: "N/A"
        }
        if state == 0:
            color='#b4dea0'
            tcolor='#323232'
        elif state == 1:
            color='#fbd646'
            tcolor='#323232'
        elif state == 2:
            color='#d95f5a'
            tcolor='#323232'
        elif state == 3:
            color='#f4bb84'
            tcolor='#323232'
        else:
            color='#ececec'
            tcolor='#323232'
        return color, tcolor, sdict[state]


@dashlet_registry.register
class BubbleDashlet(DashStatelets):
    """Dashlet that displays a static text"""
    @classmethod
    def type_name(cls):
        return "bubble"

    @classmethod
    def title(cls):
        return _("Status bubble")

    @classmethod
    def description(cls):
        return _("Displays the highest state of given group.")

    @classmethod
    def sort_index(cls):
        return 100

    @classmethod
    def vs_parameters(cls):
        return Dictionary(
            title=_('Properties'),
            render='form',
            optional_keys=["contact_group","host_group","service_group"],
            elements=cls._parameter_elements,
        )

    @classmethod
    def _parameter_elements(cls):
        elements = [
            ("contact_group",  Tuple(
                title = _('Contact group'),
                help = _('Choose the contact group you like to use.'),
                elements = [
                    ContactGroupSelection(title=_("Contact group")),
                    DropdownChoice(
                        title = _('Datasource'),
                        help = _('Choose the host group you like to use.'),
                        choices = [
                            ("eventconsolestatus", "Event Console"),
                            ("hosts", "Hosts"),
                            ("services", "Services"),
                        ],
                    ),
                    Checkbox(
                        title = _('Ignore acknowledged'),
                        label = _('Ignore acknowledged for counting'),
                        default_value = True,
                    ),
                ],
            )),
            ("host_group",  Tuple(
                title = _('Host group'),
                help = _('Choose the host group you like to use.'),
                elements = [
                    HostGroupSelection(title=_("Host group")),
                ],
            )),
            ("service_group",  Tuple(
                title = _('Service group'),
                help = _('Choose the service group you like to use.'),
                elements = [
                    ServiceGroupSelection(title=_("Service group")),
                ],
            )),
            ("width", Integer(
                title = _('Width (Pixel)'),
                default_value = 100,
                minvalue = 5,
            )),
            ('tsize', DropdownChoice(
                title = _('Text size'),
                help = _('Choose the text size you like to use.'),
                choices = [('5px','5 PX'),('10px','10 PX'),('15px','15 PX'),('20px','20 PX'),('40px','40 PX')],
            )),
        ]
        return elements

    def show(self):
        bubble_id = "dashlet_%d" % self._dashlet_id
        cg = self._dashlet_spec.get("contact_group", "")
        hg = self._dashlet_spec.get("host_group", "")
        sg = self._dashlet_spec.get("service_group", "")
        w = self._dashlet_spec.get("width", "")
        ts = self._dashlet_spec.get("tsize", "")
        color='#00BB33'
        if (cg and hg) or (cg and sg) or (hg and sg):
            html.write(_("Please select only one source!"))
        else:
            if cg:
                color, tcolor, t = self._colorpicker(self._get_grp_state_ec(cg))
            elif hg:
                color, tcolor, t = self._colorpicker(self._get_grp_state_hg(hg[0]))
            elif sg:
                color, tcolor, t = self._colorpicker(self._get_grp_state_sg(sg[0]))

            html.open_div(class_="bubble")
            html.write("<div align=center>")
            html.write('<canvas width=%d height=%d id="%s_bubble"></canvas>' %
                    (w, w, bubble_id))
            html.write("</div>")
            html.javascript("""
                var canvas = document.getElementById('%s');
                var context = canvas.getContext('2d');
                var centerX = canvas.width /2 ;
                var centerY = canvas.height /2;
                var width = %d / 2;
                var radius = width - 5;

                context.beginPath();
                context.arc(width, width, radius, 0, 2 * Math.PI, false);
                context.fillStyle = '%s';
                context.fill();
                context.lineWidth = 1;
                context.strokeStyle = '%s';
                context.stroke();
                context.font="%s Helvetica";
                context.textAlign="center";
                context.textBaseline = "middle";
                context.fillStyle = "%s";
                context.fillText("%s",(width),(width));

            """ % (bubble_id + "_bubble" , w  , color , color, ts, tcolor, t))
            html.close_div()

    @classmethod
    def styles(cls):
        return """
            div.dashlet_inner div.bubble {
            width: 100%;
            height: 100%;
        }"""

@dashlet_registry.register
class RectDashlet(DashStatelets):
    """Dashlet that displays a static text"""
    @classmethod
    def type_name(cls):
        return "rect"

    @classmethod
    def title(cls):
        return _("Status rectangle")

    @classmethod
    def description(cls):
        return _("Displays the highest state of given group.")

    @classmethod
    def sort_index(cls):
        return 100

    @classmethod
    def vs_parameters(cls):
        return Dictionary(
            title=_('Properties'),
            render='form',
            optional_keys=["contact_group","host_group","service_group"],
            elements=cls._parameter_elements,
        )

    @classmethod
    def _parameter_elements(cls):
        elements = [
            ("contact_group",  Tuple(
                title = _('Contact group'),
                help = _('Choose the contact group you like to use.'),
                elements = [
                    ContactGroupSelection(title=_("Contact group")),
                    DropdownChoice(
                        title = _('Datasource'),
                        help = _('Choose the host group you like to use.'),
                        choices = [
                            ("eventconsolestatus", "Event Console"),
                            ("hosts", "Hosts"),
                            ("services", "Services"),
                        ],
                    ),
                    Checkbox(
                        title = _('Ignore acknowledged'),
                        label = _('Ignore acknowledged for counting'),
                        default_value = True,
                    ),
                ],
            )),
            ("host_group",  Tuple(
                title = _('Host group'),
                help = _('Choose the host group you like to use.'),
                elements = [
                    HostGroupSelection(title=_("Host group")),
                ],
            )),
            ("service_group",  Tuple(
                title = _('Service group'),
                help = _('Choose the service group you like to use.'),
                elements = [
                    ServiceGroupSelection(title=_("Service group")),
                ],
            )),
            ("heigth", Integer(
                title = _('Heigth (Pixel)'),
                default_value = 100,
                minvalue = 5,
            )),
            ("width", Integer(
                title = _('Width (Pixel)'),
                default_value = 100,
                minvalue = 5,
            )),
            ("text", TextUnicode(
                title = _('Box text'),
                default_value = 'text',
            )),
            ('tsize', DropdownChoice(
                title = _('Text size'),
                help = _('Choose the text size you like to use.'),
                choices = [('5px','5 PX'),('10px','10 PX'),('15px','15 PX'),('20px','20 PX'),('40px','40 PX')],
            )),
        ]
        return elements

    def show(self):
        rect_id = "dashlet_%d" % self._dashlet_id
        cg = self._dashlet_spec.get("contact_group", "")
        hg = self._dashlet_spec.get("host_group", "")
        sg = self._dashlet_spec.get("service_group", "")
        h = self._dashlet_spec.get("heigth", "")
        w = self._dashlet_spec.get("width", "")
        t = self._dashlet_spec.get("text", "")
        ts = self._dashlet_spec.get("tsize", "")
        color='#00BB33'
        tcolor='#FFFFFF'
        if (cg and hg) or (cg and sg) or (hg and sg):
            html.write(_("Please select only one source!"))
        else:
            if cg:
                color, tcolor = self._colorpicker(self._get_grp_state_ec(cg))[:-1]
            elif hg:
                color, tcolor = self._colorpicker(self._get_grp_state_hg(hg[0]))[:-1]
            elif sg:
                color, tcolor = self._colorpicker(self._get_grp_state_sg(sg[0]))[:-1]
            else:
                color, tcolor = self._colorpicker(99)[:-1]

            html.open_div(class_="rect")
            html.write("<div align=center>")
            html.write('<canvas width=%d height=%d id="%s_rectangle"></canvas>' %
                    (w, h, rect_id))
            html.write("</div>")
            html.javascript("""
                var canvas = document.getElementById('%s');
                var rectWidth = canvas.width;
                var rectHeight = canvas.height;
                var rectWidth = %d;
                var rectHeight = %d;
            
                var ctx = canvas.getContext('2d');
                function roundRect(ctx, x, y, width, height, radius, fill, stroke) {
                    if (typeof stroke == "undefined" ) {
                        stroke = true;
                    }
                    if (typeof radius == "undefined") {
                        radius = 2;
                    }
                    ctx.beginPath();
                    ctx.moveTo(x + radius, y);
                    ctx.lineTo(x + width - radius, y);
                    ctx.quadraticCurveTo(x + width, y, x + width, y + radius);
                    ctx.lineTo(x + width, y + height - radius);
                    ctx.quadraticCurveTo(x + width, y + height, x + width - radius, y + height);
                    ctx.lineTo(x + radius, y + height);
                    ctx.quadraticCurveTo(x, y + height, x, y + height - radius);
                    ctx.lineTo(x, y + radius);
                    ctx.quadraticCurveTo(x, y, x + radius, y);
                    ctx.closePath();
                    ctx.fill();
                }
                ctx.fillStyle = "%s";
                roundRect(ctx, 0, 0, rectWidth, rectHeight, 2, true);
                ctx.font="%s Helvetica";
                ctx.textAlign="center";
                ctx.textBaseline = "middle";
                ctx.fillStyle = "%s";
                ctx.fillText("%s",(rectWidth/2),(rectHeight/2));
            """ % (rect_id + "_rectangle", w, h, color , ts, tcolor, t ))
            html.close_div()

        @classmethod
        def styles(cls):
            return """
            div.dashlet_inner div.rect {
                width: 100%;
                height: 100%;
            }"""

